function setup=doublewell(setup)
% Wave packets in 1-D doublewell potential
% Exporting to jpeg for off-line processing to produce a video-clip
%
% Usage: [setup] = doublewell([setup]) 
%   
% Both input and output are optional. If called with no arguments, a
% default is supplied Notice that an initial stage where the
% w.f. decays toward the ground state is performed through imaginary
% time evolution.  
% Source: Solution of the Schrödinger equation by a spectral method ☆
% M.D Feit, J.A Fleck Jr., A Steiger, Journal of Computational Physics
% Volume 47, Issue 3, September 1982, Pages 412–433
% P.Maraner, G.P.Tecchiolli and E.O., J.Comput.Appl.Math.37(1991)209
%
% © E. Onofri, Universita` di Parma, 2011-2014

% Begin user interface 
    
  entries={'Size',...
           'Grid',...
           'mass',...
           'time',...
	   'snapshots',...
	   'ErMode',...
	   'initMom',...
	   'x_0',...
	   '\sigma',...
           'potential strength',...
           'refine',...
           'aux parameter',...
           'euclidean steps',...
           'save snapshots?'...
           'movie prefix'
	  };
  
  
  if(nargin<1)
      default={'6.0',...   % Size
               '1024',...  % NGrid
               '1.0',...   % mass
               '1000.0',...   % time
               '1000',...     % snapshots
               'normal',...   % EraseMode 
               '0.0',...  % k0
               '2.1',...  % x_0
               '0.08',... % sigma
               '5.',...   % V_0
               '64',...
               '2.0',...
               '200',...
               'n',...
               'db_'
              };
      
  else
      default=setup;
  end
  
  heading='Initial data';
  lines=1;
  AddOpts.Resize = 'on';
  AddOpts.WindowStyle = 'normal';
  AddOpts.Interpreter = 'tex';
  
  setup=inputdlg(entries,heading,lines,default,AddOpts);
  
  L                = str2double(setup{1});
  N                = str2double(setup{2});
  mass             = str2double(setup{3});
  t1               = str2double(setup{4});
  snapshots        = str2double(setup{5});
  plotmode         = setup{6};
  mom              = str2double(setup{7});
  x0               = str2double(setup{8});
  sigma            = str2double(setup{9});
  V0               = str2double(setup{10});
  refine           = str2double(setup{11});
  modes            = [{'normal'},{'xor'},{'none'}];
  omega            = str2double(setup{12});
  EuSt             = str2double(setup{13});
  movie            = setup{14};
  moviename        = setup{15};
  
  if(nnz(strcmp(modes,plotmode))==0),
      disp(['Unknown mode: ' plotmode]);
      disp('Legal modes: ')
      disp(modes);
      plotmode= 'normal';
  end
  
  %--------------------------- end user interface 
  %%----------------------------  coordinate grid  
  dx  = 2*L/N
  x   = -L*(1-1/N) : dx : L*(1-1/N); 
  x   =  x';
  
  %%----------------------------  momentum grid
  k = fftshift(pi*(-N/2:(N/2-1))/L)';
  k2 = k.^2;
  tau = t1/snapshots;
  t = tau;
  k2 = min(k2, 2* mass* pi *refine/tau); % antialias trick
  
  %
  %% Insert potential here 
  V = V0 * (1-(x/omega).^2).^2; % V0 is the height of the
                                % central barrier
  V = min(V, pi*refine/tau);    % antialias trick
  
  U = exp(-1i*tau*V/refine);
  W = exp(-0.5i*k2/mass*tau/refine);
  U2 = exp(-0.5i*tau*V/refine);

  %%--- "Euclidean" evolution - used to prepare
  %---- the state close to the approximate ground state 
  
  EU = exp(-tau*V/refine);
  EW = exp(-0.5*k2/mass*tau/refine);
  EU2 = exp(-0.5*tau*V/refine);

  % prepare initial wfcn by evolution in imaginary time: psi->exp(-T H)*psi

  psi = exp(-(x-x0).^2/4/sigma+1i*mom*x)/(2*pi*sigma).^(1/4);

  psi=psi.*EU2;

  for j=1:EuSt,
      psi = EU.*ifft(EW.*fft(psi));
  end
  
  psi=psi./EU2;
  
  a   = x(2)-x(1);
  psi = psi/(sqrt(a)*norm(psi));     % normalizzazione ||psi||=1
  %%-----------------------------------prepare the plot
  scrsz = get(0,'ScreenSize');
  fig = figure('Position',[1, 1, scrsz(3)/2, scrsz(4)/2])
  whitebg([.4,.4,.2])
  set(fig,'DoubleBuffer','on','Windowstyle','docked');
  set(gca,'NextPlot','replace','Visible','off')
  
  area(x,V/V0*max(abs(psi))^2/4,'Facecolor',[0.25,.25,.5], ...
       'EdgeColor','r', 'LineStyle','none');
  
  hold on
  
  plot(x, abs(psi).^2, 'w','LineWidth', 0.50);
  
  xlabel('\fontsize{16} x'); ylabel('\fontsize{16}|\psi |^2');
  
  title(['\it Schroedinger Wave packet evolution in a double-well ' ...
         '' ']potential'], 'FontSize',18, 'EdgeColor','w', ...
        'Backgroundcolor',[.64,.65,.99]);
  
  axis([min(x), max(x), 0, max(abs(psi).^2)]);
  
  set(gca,'Color',[.64,.65,.99])
  
  h=area(x, abs(psi).^2,'Facecolor','y', 'EraseMode',plotmode, ...
         'LineStyle','none')
  
  set(gcf,'DoubleBuffer','on');

  h1 = findobj(gca,'Type','patch');

  set(h1,'FaceAlpha',0.5);

  Ep=sum(V.*abs(psi).^2)/sum(abs(psi).^2); % compute total energy

  phi=fft(psi);

  Ek=sum(k2.*abs(phi).^2/(2*mass))/sum(abs(phi).^2);
  
  E=Ep+Ek;
  E=E/V0*max(abs(psi))^2/4;              % stessa normalizzazione
                                         % del plot di V(x) 
  plot([x(1),x(end)],E*[1,1],'r');
  
  
  %%------------------------------- main loop
  n=1;
  while(t<t1),
    set(h,'Ydata',abs(psi).^2);
    drawnow expose;
    psi=psi.*U2;
    for j=1:refine
      psi = U.*ifft(fft(psi).*W);
    end
    psi=psi.*conj(U2);
    home;
    n=n+1;
    t=t+tau;
    clc 
    home
    disp(['t=',num2str(t)]);
    if(movie=='y')   % yes, generate a sequence of snapshots
       ver=sprintf('%d',1000000+n);
       print('-djpeg',['images/',moviename,ver]);
    end
  end
  
  hold off;
  
