function [CG,Tab] = clebsch(j1,j2,epsilon)
% clebsh.m
% somma di momenti angolari
% coefficienti di Clebsh-Gordan
%
% usage: [CG,Tab] = clebsh(j1,j2,epsilon)
%
% epsilon viene utilizzato per individuare la matrice non-degenere
% da diagonalizzare: invece di diagonalizzare J^2 si opera su
% J^2+epsilon Jz in modo da forzare la base a essere la corretta
% base della somma di momenti angolari. Provare con epsilon = 0 e
% ~=0. Epsilon deve essere scelto in modo tale da evitare
% degenerazione. 
% NB!!!
% CG contiene i coefficienti di Clebsh-Gordan in formato di matrice
% sparsa. Tab le informazioni su m1,m2,J,M. La convenzione \`e
% quella di PDG (cioe` sign(CG) * (CG)^2, quindi ad es. 
% -2/3 e` da leggere -sqrt(2/3) 
%
% Copyright  2004-2013 by E.Onofri

if nargin<3, epsilon = .0001;end
if nargin<2,j2 = 1/2; end
if nargin<1,j1 = 1/2; end

more on
m1 = -j1:j1;
m2 = -j2:j2;
n1 = 2*j1+1;
n2 = 2*j2+1;
  
  d1p = ((j1-m1).*(j1+m1+1)).^(1/2);
  d1m = ((j1+m1).*(j1-m1+1)).^(1/2);
  d2p = ((j2-m2).*(j2+m2+1)).^(1/2);
  d2m = ((j2+m2).*(j2-m2+1)).^(1/2);
  
  Jp1 = spdiags(d1p',-1,n1,n1);
  Jp2 = spdiags(d2p',-1,n2,n2);

  Jm1 = spdiags(d1m',1,n1,n1);
  Jm2 = spdiags(d2m',1,n2,n2);
  
  Jx1 = (Jp1+Jm1)/2;
  Jy1 = (Jp1-Jm1)/(2*i);            
  Jz1 = spdiags(m1',0,n1,n1);
  Jx2 = (Jp2+Jm2)/2;
  Jy2 = (Jp2-Jm2)/(2*i);  
  Jz2 = spdiags(m2',0,n2,n2);
  
  one1 = eye(n1);
  one2 = eye(n2);

  Jx1 = kron(Jx1,one2);
  Jy1 = kron(Jy1,one2);
  Jz1 = kron(Jz1,one2);
  Jx2 = kron(one1,Jx2);
  Jy2 = kron(one1,Jy2);
  Jz2 = kron(one1,Jz2);
  
  
  Jx = Jx1+Jx2;
  Jy = Jy1+Jy2;
  Jz = Jz1+Jz2;
  
  Jp = Jx+i*Jy;
  Jm = Jx-i*Jy;
  
  J2 = Jx^2+Jy^2+Jz^2;
  
  [Psi,J] = eig(full(J2));
  
  J = diag(J)';
  
  ind = find(abs(diff(J))>epsilon);
  disp(' ')
  
  disp(['j1,j2:',rats(j1),' x ',rats(j2),'  =>  ',rats((-1/2+ sqrt(1/4+J([ind, end]))))])
  disp(' ')
  
  [Psi,J] = eig(full(J2+epsilon*Jz));

  ind = find(abs(Psi)<1.e4*eps);
  Psi(ind) = 0;
  CG = sparse(sign(Psi).*Psi.^2);

  N = (2*j1+1)*(2*j2+1);
  
  TD = [diag(full(Jz1)),diag(full(Jz2)),(1:N)'];
  TP = [(diag(Psi\J2*Psi)+1/4).^.5-1/2, diag(Psi\Jz*Psi),(1:N)'];
  
  [a,b] = find(CG);
  
  for k = 1:length(a)
    Tab{a(k),b(k)}  = [TD(a(k),1),TD(a(k),2),...
		     TP(b(k),1),TP(b(k),2)];
  end
  disp(['      m1             m2            J             M' ...
	'    <JM|j1 j2 m1 m2>'])
  
  for k = 1:length(a)
    disp([rats(Tab{a(k),b(k)}), rats(CG(a(k),b(k)))]);
  end
  
  more off