function [k, delta, setup] = phaseshifts(default, batch)
% Si risolve l'equazione radiale nel range di interazione e si
% estrapola alla zona asintotica con una combinazione di soluzioni
% libere da cui si estrae lo sfasamento
% Usage:
%    [k,delta, setup]= phaseshifts(setup, batch)
% setup contains all parameters which show up in the dialog box
% Units: Planck's cnst = 1, mass = 1/2
% if 'batch' is nonnull, program does not invoke inputdlg
%-------------------- E.Onofri, M. Rossi, 2012 -------------------------------

    entries = {'r_{max}',...            % cutoff su raggio
               'ell',...                % momento angolare
               'tol'...                 % accuratezza richiesta  a ode113
               'V_0',...                % coupling
               '\eta',...               % Yukawa parameter
               'hard core',...          % 
               '[k_min, k_max] (waves/nm)',... % wave number interval
               'number of k',...        % number of points in k_grid
               'save <y/n>',...
               'save to...'
              };

    if nargin<1,
        default = {'20',... % size
                   '0',...
                   '1.e-8',...
                   '-1.',...
                   '.5',...
                   '.005',...
                   '[.1, 1.]',...
                   '200',...
                   'n',...
                   'phash_0'
                  };
    end
    
    AddOpts.Resize  =  'on';
    AddOpts.WindowStyle  =  'normal';
    AddOpts.Interpreter  =  'tex';

    if nargin<2,
        setup = inputdlg(entries,'parameters',1,default,AddOpts);
    else
       setup=default;
    end

    rmax = str2num(setup{1});
    ell  = str2num(setup{2});
    tol  = str2num(setup{3});
    V0   = str2num(setup{4});
    eta  = str2num(setup{5});
    hc   = str2num(setup{6});
    k_grid = str2num(setup{7});
    nk = str2num(setup{8});
    fout   = setup{10};
    
    k_min=k_grid(1);
    k_max=k_grid(2);
    %    k_grid=logspace(log(k_min)/log(10),log(k_max)/log(10),nk);
    k_grid = linspace(k_min,k_max,nk);
    
    cumdelta = [];
    
    opt = odeset('RelTol',tol,...
                 'AbsTol',tol,...
                 'refine',2);

    %  'OutputFcn','odeplot',...
    %  'OutputSel',1,...
    
    r_grid = linspace(hc,rmax,128);
    h = waitbar(0,'Please wait...');
    count=0;
    for k = k_grid, % scanning momentum space
       count=count+1;
       waitbar(count/nk);
       
       [r,psi]=ode113(@schr,r_grid,[0,1],opt); % solve radial Schr.Eq.

        %% WARNING: Attenzione: si lavora sulla funzione radiale e le 
        % soluzioni libere sono le Bessel sferiche moltiplicate per r,
        % da qui la formula per il raccordo liscio tra soluzione
        % interna ed esterna. Questo richiede il termine
        % "-jn*psi(end,1)..."
        
        
        [jn,yn,djn,dyn]=sphbessel(ell,k*rmax); % external solution
        
        tdelta =(jn*psi(end,2)-k*djn*psi(end,1)-jn*psi(end,1)/rmax)/...
                (yn*psi(end,2)-k*dyn*psi(end,1)-yn*psi(end,1)/rmax); 
        
        
        
        delta = atan(tdelta);               % phase shift
        cumdelta = [cumdelta, delta];
    
    end
    
    close(h);
    
    k = k_grid;
    delta = cumdelta;

    %    hold off
    
    % plot3(cos(2*delta),sin(2*delta),k)
    % set(gcf,'WindowStyle','docked')
    % xlabel('cos(2\delta)','FontSize',14)
    % ylabel('sin(2\delta)','FontSize',14)
    % zlabel('k','FontSize',14)
        
    subplot(211)
    hpl=plot(k_grid.^2, 4*pi*sin(delta).^2./k_grid.^2,'.-');
    set(gcf,'WindowStyle','docked')
    title(['\sigma_{',num2str(ell),'}'],'FontSize',14)
    xlabel('k^2','FontSize',14)
    grid on
    
    subplot(212)
    plot(r,pot(r) + ell*(ell+1)./r.^2,'g','LineWidth',2)
    grid on
    
    eval(['save ' fout])
    
%%---------- Schroedinger eq. -----------------
    function dpsi = schr(r,psi)
        dpsi=[psi(2);
              (pot(r) + ell*(ell+1)./r.^2 -k^2).*psi(1)];
    end
    
%% Spherical Bessel fcts - Matlab provides only cylindrical Bessel

function [jn,yn,djn,dyn]=sphbessel(n,x)
   pref = sqrt(pi./(2*x));
   jn = pref.*besselj(n+1/2,x);
   yn = pref.*bessely(n+1/2,x);
   
   % derivatives needed in matching at r=Rmax
   djn = pref.*besselj(n-1/2,x) - (n+1)./x.*jn;
   dyn = pref.*bessely(n-1/2,x) - (n+1)./x.*yn;
end

%% ------------ Potential energy -----------------
function V=pot(r)
% V = V0*exp(-eta*r)./r;      % Yukawa potential
% V = V0*(rmax-r)/rmax; % conical barrier
% V = V0;         % square barrier
% V = V0*exp(-eta*r)./(1-exp(-eta*r));
% V = V0*exp(-eta*r).*(r.^8-10);
   V = V0*(r<rmax/16) + eta*(r>rmax/16)./r;
end

end