function [E0, gamma, amp] = resonance_fit(E, sigma, initial_params)
%RESONANCE_FIT Finds the resonance peak using the Breit-Wigner distribution
% Usage:
%       [E0, gamma, amp] = RESONANCE_FIT(E, sigma, initial_params)
%       Returns the energy, width and amplitude of the resonance peak.
%       E and sigma are the data vectors. initial_params is an
%       optional 3-element vector containing an initial guess for E0,gamma,
%       amp

% Breit-Wigner distribution:
%          a /( (E-E0)^2  + (gamma/2)^2)

    global DEBUG

    if nargin < 3 %Parametri di default
        initial_params = [E(end)/2,.01,1];
        %initial_params = rand(1,3);
    end

    if DEBUG
        options = optimset('Display','iter');
    else
        options = optimset;
    end

    %Ricerca del minimo
    result = fminsearch(@sse_brwig,initial_params,options,E,sigma);

    E0 = result(1);
    gamma = result(2);
    amp = result(3);
    
    E_fit = linspace(E(1),E(end),500);
    
    bw = breit_wigner(result,E_fit);
    
    [SSE,r] = sse_brwig(result,E,sigma);
    rsq = 1 - sum(r.^2) / sum((sigma - mean(sigma)).^2);

    % To check the fit
    plot(E,sigma,'*')
    hold on
    plot(E_fit,bw,'r')
    hold off

    text(.75*E(end),.9*max(sigma),['R^2 = ',num2str(rsq)]);
    xlabel('E')
    ylabel('\sigma')
    legend('Data','Breit-Wigner fit');

end

function bw = breit_wigner(params,E)
    E0 = params(1);
    gamma = params(2);
    A = params(3);

    %Breit-Wigner distr
    bw = A.*(gamma).^2/4./((E-E0).^2+(gamma).^2/4);
end

%Funzione somma dei residui al quadrato da minimizzare
function [SSE, Err] = sse_brwig(params,E,sigma)
    bw = breit_wigner(params,E);
    
    Err = bw - sigma;
    SSE = Err(:)'*Err(:);

end