function [E, a1,a2] = Boson2D(mode, lambda, N, neig)
% Build and diagonalize the Hamiltonian for a two-dim oscillator
% with a simple perturbation 
% Creation-annihilation operators are built via Kronecker products 
% V contains the perturbation: default V = a1 a2' + h.c.
% (a' in matlab is the Hermitian conjugate)
%
% Usage: [E, a1, a2] = Boson2D(mode, lambda, N, neig)
% mode='1'--> perturbation is  a1'*a2 + a2'*a1
%     ='2'-->        "         a1'*a2^2 + ...
%     ='3'-->        "        (a1'^2 + a2'^2)*(a1^2 + a2^2)
% lambda is the coupling
% N is the maximum occupation number in each oscillator
% neig is the number of eigenvalues to be computed
% 
% Copyright (C) 2013 by E. Onofri

if nargin<4, neig = 64; end
if nargin<3, N = 32; end
if nargin<2, lambda = 0.05; end
if nargin<1, mode='1'; end
a = boson(N);
Id = speye(size(a));
a1 = kron(a,Id); % a1 = a ⊗ 1 
a2 = kron(Id,a); % a2 = 1 ⊗ a 

opts.tol    = 1e-12; % to be used by eigs 
opts.disp   = 0;    % eigs works quietly - alternatively =1 , =2.
opts.isreal = true;
opts.issym  = true;

H0 = a1'*a1 + a2'*a2; % unperturbed Hamiltonian

switch mode
  case '1'
    V = a1'*a2;      
    V = V + V';
  case '2'
    V = (a1'*a2^2 + a2'*a1^2);
    V = V + V';
  case '3'
    V = (a1^2 + a2^2)'*(a1^2 + a2^2); 
  otherwise
    disp('unrecognised mode');
    return
end

H = H0 + lambda *V;

clear H0, V;                       % save memory

E = eigs(H, neig, 'SA', opts);
stem(E,'fill')

%-------------------------- Creation/annihilation operators--------
function [a,ad] = boson(B)

% building the sparse matrices 
D  = (0:B)';                   % diagonal mat.elem. (column vector)
a = spdiags(sqrt(D), 1, B, B); % creation operator

if nargout>1, ad = a'; end
