function [E, a1,a2,a3,a4] = Boson4D(mode, lambda, N, neig)
% Build and diagonalize the Hamiltonian for a four-dim oscillator
% with a simple perturbation as a function of creation and
% annihilation op.s 
% Cr/annih. operators are built via Kronecker products 
% V contains the perturbation: default V = a1 a2' + permut. + h.c.
% (a' in matlab is the Hermitian conjugate)
%
% Usage: [E, a1, a2, a3, a4] = Boson4D(mode, lambda, N, neig)
%
% mode='1'--> perturbation is  a1'*a2 + a2'*a3 + a3'*a1
%     ='2'-->        "         a1'*a2^2 + a2'*a3^2 + a3'*a1^2; 
%     ='3'-->        " (a1'^2 + a2'^2 + a3'^2)*(a1^2 + a2^2 + a3^2)
% lambda is the coupling H=H0+lambda*V
% N is the maximum occupation number in each oscillator
% neig is the number of eigenvalues to be computed
% 
% Copyright (C) 2013 by E. Onofri

if nargin<4, neig = 64; end
if nargin<3, N = 32; end
if nargin<2, lambda = 0.05; end
if nargin<1, mode='1'; end

% Build the canonical operators in Fock space
a = boson(N);
Id = speye(size(a)); %... using Kronecker products
a1 = kron(a,kron(Id,kron(Id,Id))); % a1 = a ⊗ 1 ⊗ 1 ⊗ 1
a2 = kron(Id,kron(a,kron(Id,Id))); % a2 = 1 ⊗ a ⊗ 1 ⊗ 1
a3 = kron(Id,kron(Id,kron(a,Id))); % a3 = 1 ⊗ 1 ⊗ a ⊗ 1
a4 = kron(Id,kron(Id,kron(Id,a))); % a3 = 1 ⊗ 1 ⊗ 1 ⊗ a

opts.tol    = 1e-12; % to be used by eigs 
opts.disp   = 0;     % let's eigs work quietly - alternatively =1 , =2.
opts.isreal = true;
opts.issym  = true;

H0 = a1'*a1 + a2'*a2 + a3'*a3 + a4'*a4; % unperturbed Hamiltonian

switch mode
  case '1'
    V = a1'*a2 + a2'*a3 + a3'*a4 + a4'*a1;       % define V ...
    V = V + V';                         % ... and announce it
    disp([' V = a1''','*a2 + a2''','*a3 + a3''','*a4 + ... + h.c.']); 
                                        
  case '2'
    V = a1'*a2^2 + a2'*a3^2 + a3'*a4^2 + a4'*a1^2;
    V = V + V';
    disp(['V=a1''','*a2^2 + a2''','*a3^2 + a3''','*a4^2 +...+ h.c.'])
  case '3'
    V = (a1^2 + a2^2 + a3^2 +a4^2)'*(a1^2 + a2^2 + a3^2 + a4^2); 
    disp(['V=(a1^2 + a2^2 + a3^2 + a4^2)''','*(a1^2 + a2^2 + a3^2+ a4^2)']);
  otherwise
    disp('unrecognised mode');
    return
end

H = H0 + lambda * V;

clear H0, V;                       % save memory

E = eigs(H, neig, 'SA', opts);
stem(E,'fill')

%-------------------------- Creation/annihilation operators--------
function [a,ad] = boson(B)

% building the sparse matrices 
D  = (0:B)';                   % diagonal mat.elem. (column vector)
a = spdiags(sqrt(D), 1, B, B); % creation operator

if nargout==2, ad = a'; end
