function E = Quartic3D(N, lam, neig)
% Build and diagonalize the Hamiltonian for three-dim oscillator with
% a simple quartic perturbation 
% Creation-annihilation operators are built via Kronecker products
% V contains a quartic perturbation
% Default V=(a1^2+ a2^2+ a3^2)'*(a1^2+ a2^2+ a3^2)
%
% Usage: [H,E]=Quartic3D(N, lam, neig)
% N is the maximum occupation number in each oscillator
% lam is the coupling
% neig is the number of eigenvalues to be computed
%
% Copyright (C) 2013 by E. Onofri

if nargin<3, neig = 64; end
if nargin<2, lam = 2/3; end
if nargin<1, N = 32; end

a  = boson(N);
Id = speye(size(a));

% Use "kron" to build a three-dim array

a1 = kron(a,kron(Id,Id)); % a1 = a ⊗ 1 ⊗ 1
a2 = kron(Id,kron(a,Id)); % a2 = 1 ⊗ a ⊗ 1
a3 = kron(Id,kron(Id,a)); % a3 = 1 ⊗ 1 ⊗ a

opts.tol  = 1e-12; % to be used by eigs 
opts.disp = 0;     % eigs works quietly - alternatively =1 , =2.
opts.isreal = true;
opts.issym  = true;

H0 = a1'*a1 + a2'*a2 + a3'*a3;
V = (a1'^2 + a2'^2 + a3'^2)*(a1^2 + a2^2 + a3^2);
H = H0 + lam * V/4;
clear a1 a2 a3 V;

E = eigs(H, neig, 'SA', opts);
stem(E,'fill');
%-------------------------- Creatori e annichilatori 3D --------
function a = boson(B)

% building the sparse matrices 
D  = (0:B-1)';                 % diagonal m.e. (column vector)
a = spdiags(sqrt(D), 1, B, B); % annihilation operator
ad=a';

