function E = Fock3D(mode, lam, N)
% Oscillatore 3D con perturbazione a_1 a_2^+ + permutazioni + h.c.
% Creations/annihilation operators are built as sparse matrices
% acting on threedim arrays <n1,n2,n3| psi> via circshift with
% correction at the border to avoid periodicity artifacts (the goal
% is a shift with zero-padding)
% See companion code "Boson3D.m" where the canonical operators are
% built via Kronecker products.
% 
% Usage: E = Fock(mode, lam, N) 
% where: N(def.=3) max occupation number for each d.o.f
%        lam(def.=.1) coupling
%        mode: '1' --> exactly solvable case V = a1' a2 + a2' a3 +
%        a3' a1 + herm. conj. (default)
%              '2' --> V = a1^2 a2' + a2^2 a3' + a3^2 a1'
%              +herm. conj.
%              '3' --> V = a1^2 a2'^2 + a2^2 a3'^2 + a3^2 a1'^2
%              +herm. conj.
%
% Copyright © 2013 by E.Onofri
   
   if nargin<3,  N = 3;end
   if nargin<2,lam = .1;end
   if nargin<1,mode = '1';end
   
   [n2,n1,n3] = meshgrid(0:N,0:N,0:N);
   opts.tol  = 1e-12; % to be used by eigs 
   opts.disp = 0;    % eigs works quietly - alternatively =1 , =2.
   opts.isreal = true;
   opts.issym  = true;
   
   m = 1:N;
   neig = sum((m+1).*(m+2))/2;
   disp(['Computing ',num2str(neig),' eigenvalues']);

   switch mode
     case '1'
       E = eigs(@pot1,(N+1)^3,neig,'SA',opts);
     case '2'
       E = eigs(@pot2,(N+1)^3,neig,'SA',opts);
     case '3'
       E = eigs(@pot3,(N+1)^3,neig,'SA',opts);
     otherwise
       disp('unknown mode-legal choices: 1-2')
   end
   stem(E)
%%%%%%
function Hpsi = pot1(psi)                % compute the action of the Hamiltonian in Fock space
   psi = reshape(psi,N+1,N+1,N+1);      % reshape to NxNxN array
   
   Vpsi1 = sqrt(n1.*(n2+1)).*circshift(psi,[1,-1, 0]);
   Vpsi1(:,N+1,:) = 0;
   
   Vpsi2 = sqrt(n2.*(n1+1)).*circshift(psi,[-1,1,0]);
   Vpsi2(N+1,:,:) = 0;
   
   Vpsi3 = sqrt(n2.*(n3+1)).*circshift(psi,[0,1,-1]);
   Vpsi3(:,:,N+1) = 0;
   
   Vpsi4 = sqrt(n3.*(n2+1)).*circshift(psi,[0,-1,1]);
   Vpsi4(:,N+1,:) = 0;

   Vpsi5 = sqrt(n3.*(n1+1)).*circshift(psi,[-1,0,1]);
   Vpsi5(N+1,:,:) = 0;
   
   Vpsi6 = sqrt(n1.*(n3+1)).*circshift(psi,[1,0,-1]);
   Vpsi6(:,:,N+1) = 0;
   
   Hpsi = (n1+n2+n3).*psi + lam*(Vpsi1+Vpsi2+Vpsi3+Vpsi4+Vpsi5+Vpsi6);
   Hpsi = reshape(Hpsi,(N+1)^3,1,1);
end

%%%%%% Example of non-bilinear-perturbation
function Hpsi = pot2(psi)                % compute the action of the Hamiltonian in Fock space
   psi = reshape(psi,N+1,N+1,N+1);      % reshape to NxNxN array
   
   % circshift con arg positivo per annichilatore, negativo per
   % costruttore. Compensare per periodicita` nel caso di indice
   % negativo azzerando esplicitamente un numero di elementi pari alla
   % potenza - e.g. a1+^2 -> Vpsi(N:N+1,:,:)=0; 
   % a3+^3 -> Vpsi(:,:,N-1:N+1)=0;
   
   Vpsi1 = sqrt(n1.*(n1-1).*(n2+1)).*circshift(psi,[2,-1, 0]); 
   Vpsi1(:,N+1,:) = 0;
   
   Vpsi2 = sqrt(n2.*(n1+1).*(n1+2)).*circshift(psi,[-2,1,0]);
   Vpsi2(N:N+1,:,:) = 0;
   
   Vpsi3 = sqrt(n2.*(n2-1).*(n3+1)).*circshift(psi,[0,2,-1]);
   Vpsi3(:,:,N+1) = 0;
   
   Vpsi4 = sqrt(n3.*(n2+1).*(n2+2)).*circshift(psi,[0,-2,1]);
   Vpsi4(:,N:N+1,:) = 0;

   Vpsi5 = sqrt(n3.*(n3-1).*(n1+1)).*circshift(psi,[-1,0,2]);
   Vpsi5(N+1,:,:) = 0;
   
   Vpsi6 = sqrt(n1.*(n3+1).*(n3+2)).*circshift(psi,[1,0,-2]);
   Vpsi6(:,:,N:N+1) = 0;
   
   Hpsi = (n1+n2+n3).*psi + lam*(Vpsi1+Vpsi2+Vpsi3+Vpsi4+Vpsi5+Vpsi6);
   Hpsi = reshape(Hpsi,(N+1)^3,1,1);
end

%%%%%% Example of non-bilinear-perturbation
function Hpsi = pot3(psi)                % compute the action of the Hamiltonian in Fock space
   psi = reshape(psi,N+1,N+1,N+1);      % reshape to NxNxN array
   
   % circshift con arg positivo per annichilatore, negativo per
   % costruttore. Compensare per periodicita` nel caso di indice
   % negativo azzerando esplicitamente un numero di elementi pari alla
   % potenza - e.g. a1+^2 -> Vpsi(N:N+1,:,:)=0; 
   % a3+^3 -> Vpsi(:,:,N-1:N+1)=0;
   
   Vpsi1 = sqrt(n1.*(n1-1).*(n2+1).*(n2+2)).*circshift(psi,[2,-2, 0]); 
   Vpsi1(:,N:N+1,:) = 0;
   
   Vpsi2 = sqrt(n2.*(n2-1).*(n1+1).*(n1+2)).*circshift(psi,[-2,2,0]);
   Vpsi2(N:N+1,:,:) = 0;
   
   Vpsi3 = sqrt(n2.*(n2-1).*(n3+1).*(n3+2)).*circshift(psi,[0,2,-2]);
   Vpsi3(:,:,N:N+1) = 0;
   
   Vpsi4 = sqrt(n3.*(n3-1).*(n2+1).*(n2+2)).*circshift(psi,[0,-2,2]);
   Vpsi4(:,N:N+1,:) = 0;

   Vpsi5 = sqrt(n3.*(n3-1).*(n1+1).*(n1+2)).*circshift(psi,[-2,0,2]);
   Vpsi5(N:N+1,:,:) = 0;
   
   Vpsi6 = sqrt(n1.*(n1-1).*(n3+1).*(n3+2)).*circshift(psi,[2,0,-2]);
   Vpsi6(:,:,N:N+1) = 0;
   
   Hpsi = (n1+n2+n3).*psi + lam*(Vpsi1+Vpsi2+Vpsi3+Vpsi4+Vpsi5+Vpsi6);
   Hpsi = reshape(Hpsi,(N+1)^3,1,1);
end
end