function  [E, setup, Psi, x, y] = schr2D(setup)
% Calcolo dello spettro di Operatore di Schroedinger del tipo 
% H = p^2/2m + V(x,y) (due dimensioni).
% Utilizza la definizione dell'operatore attraverso la FFT
% L'energia potenziale e` determinata dalla variabile V 
% E contiene lo spettro di autovalori (solo i primi Neig)
% Psi contiene i dati sulle autofunzioni - per visualizzare la funzione
% d'onda utilizzare schr2_disp 
% Usage:
%   [E, setup, Psi, x, y] = schr2D(setup)
% Per testare la routine scegliere V=r2 -- output E=.5, 1.5, 2.5, etc

%%----- setting up parameters via "inputdlg"
    if nargin<1
       setup = initialize();
    else
       setup = initialize(setup);
    end
    
    % extract parameters converting cells to numbers where appropriate
    par      = str2num(setup{1})     ;
    lambda   = str2num(setup{2})     ;
    L        = str2num(setup{3})        ;
    N        = str2num(setup{4})         ; 
    Neig     = str2num(setup{5})      ;
    prec     = str2num(setup{6})      ;
    verbose  = str2num(setup{7})   ;
    lancz    = str2num(setup{8})     ;
    


%% -------- space Lattice ----------------------
dx = 2*L/N;
[x,y] = meshgrid(-L*(1-1/N) : dx : L*(1-1/N)); % symmetrized instead
r = sqrt(x.^2+y.^2);                         % of choosing [-L,L-a]
r2= r.^2;
%% -------- momentum lattice in Fourier space --
fourier_space = fftshift(pi*(-N/2:N/2-1)/L); 
[kx,ky] = meshgrid(fourier_space);

%% Partial derivatives in Fourier space --------

kxx = kx.^2;	                        % Partial derivatives
kyy = ky.^2;
k2 = kxx+kyy;                           % Laplacean

%% Potential Energy
V = 0.5 * par * r2 + lambda * r2.^2; % Oscillatore armonico perturbato 
                             % Other choices
                             % V = -par./r;  % Coulomb
                             % V = x.^2.*y.^2; % Simon's potential
                             % V = -par*exp(-r.^2/2/lambda)/(2*pi*lambda);

% Options to be passed to eigs 
opts.tol = prec; 
opts.disp = verbose;    % =0 eigs works quietly - alternatively =1 , =2.
opts.isreal = true;
opts.issym = true;
opts.p = lancz;
opts.v0 = reshape(exp(-0.5 * par * r2)',N^2,1);     % initial state to start Arnoldi

%% Diagonalization using "eigs"

[Psi, E] = eigs(@Hamiltonian, N^2, Neig, 'SA', opts);
E = diag(E);
stem(E,'filled')
%%--- embedded routine: compute the variation equations ---------

function xout = Hamiltonian(xin)
% Embedded function  - CAUTION! All variables above are seen by the routine
% Insert here the definition of the Hamiltonian

psi = reshape(xin(1:N^2),N,N);          % eigs works with vectors
fpsi = fft2(psi);
Deltapsi = real(ifft2(k2.*fpsi));

Hpsi  = 0.5* Deltapsi +  V .* psi;

xout   = reshape(Hpsi, N^2, 1);

end

%% get parameters from input dialog
function data = initialize(setup)
    % Options
   AddOpts.Resize      = 'on';
   AddOpts.WindowStyle = 'normal';
   AddOpts.Interpreter = 'tex';
   
   entries={'parameter'...                 % first parameter
            '\lambda'...                % second parameter
            'L'...
            'N'...
            'Neig'...
            'precisione'...
            'verbose'...
            'Lanczos vectors'
            };

   if(nargin<1)                         % valori di default
       default = {'0.5',...
                '0'...
                '10.0'...
                '64'...
                '12'...
                '1e-8'...
                '0'...
                  '20'
               };
   else
       default = setup;                      % reusing a previous setup
   end
   
    
  title = 'Schroedinger spectrum - 2 dim';
  lines = 1;
  data = inputdlg(entries,title,lines,default,AddOpts);
end
end

