function  [E, setup, Psi, x, y, z] = schr3D(setup)
% Calcolo dello spettro di Operatore di Schroedinger del tipo 
%         H = p^2/2m + V(x,y,z) 
% Utilizza la definizione dell'operatore attraverso la FFT
% L'energia potenziale e` definita dalla variabile V 
% "E" contiene lo spettro di autovalori (solo i primi Neig)
% "Psi" contiene i dati sulle autofunzioni - per visualizzare la funzione
% d'onda utilizzare schr3_disp - vengono mostrati gli orbitali
% cioe` superfici di valore costante per |psi|^2
% Per testare la routine scegliere V=par1*r2/2, par=1 -- output
% E=3/2,5/2 (triplo), 7/2 (sestuplo) etc
% Usage:
%  [E, setup, Psi, x, y, z] = schr3D(setup)
%----------------------------------------------------------------------
   
%%----- setting up parameters via "inputdlg"
    if nargin ==1,
        setup = initialize(setup);
    else
        setup = initialize();
    end
    
    % extract parameters converting cells to numbers where appropriate
    par1      = str2num(setup{1})         ;
    par2      = str2num(setup{2})         ;
    L     = str2num(setup{3})         ;
    N    = str2num(setup{4})         ;
    Neig    = str2num(setup{5})         ;
    prec    = str2num(setup{6})         ;
    verbose    = str2num(setup{7})         ;
    
    
%% -------- space Lattice ----------------------
dx = 2*L/N;
x0 = -L*(1-1/N) : dx : L*(1-1/N);
[x,y,z] = meshgrid(x0,x0,x0); % symmetrized instead
r = sqrt(x.^2+y.^2+z.^2);                         % of choosing
                                                  % [-L,L-a]
r2=r.^2;
%% -------- momentum lattice in Fourier space --
k = fftshift(pi*(-N/2:N/2-1)/L); 
[kx,ky,kz]=meshgrid(k,k,k);

%% Partial derivatives in Fourier space --------

kxx = kx.^2;	                        % Partial derivatives
kyy = ky.^2;
kzz = kz.^2;
k2 = kxx + kyy + kzz;                           % Laplacean

%% Potential Energy
% V = -par1*exp(-r.^2/2/par2)/(2*pi*par2)^1.5;
V =  0.5*par1^2 * r2 + par2*r2.^2; % Oscillatore armonico perturbato con r^4
                                   % V = par1*r;
                                   % V = -par1./r;  % Coulomb
                                   % V = par1 * x.^2.*y.^2.*z.^2; % Simon's potential

%% Options to be passed to eigs
opts.tol = prec; % to be used by eigs (JacobiSpectrum)
opts.disp = verbose;    % eigs works quietly - alternatively =1 , =2.
opts.isreal = true;
opts.issym = true;
opts.v0 = reshape(exp(-0.5 * par1 * r2),N^3,1);     % initial state to start Arnoldi
%% Diagonalization using "eigs"

[Psi, E] = eigs(@Hamiltonian, N^3, Neig, 'SA', opts);
E = diag(E);
stem(E,'filled')
%%--- embedded routine: compute the action of the Hamiltonian

function xout=Hamiltonian(xin)
% Insert here the definition of the Hamiltonian

psi = reshape(xin(1:N^3),N,N,N);          % eigs works with vectors
fpsi = fftn(psi);
Deltapsi = real(ifftn(k2.*fpsi));

Hpsi  = 0.5* Deltapsi +  V .* psi;

xout   = reshape(Hpsi, N^3, 1);

end
%%------------------------------------------
function data = initialize(setup)
    
    % Options
        AddOpts.Resize      = 'on';
        AddOpts.WindowStyle = 'normal';
        AddOpts.Interpreter = 'tex';
        
        entries={'\omega'...                 % first parameter
                 '\lambda'...                % second parameter
                 'L'...
                 'N'...
                 'Neig'...
                 'precisione'...
                 'verbose'
                };
        
        if(nargin==0)                         % valori di default
            default={'0.5',...
                     '0'...
                     '10.0'...
                     '40'...
                     '12'...
                     '1e-8'...
                     '0'            
                    };
            
        else
            default=setup;                      % reusing a previous setup
        end
        
        
        title='Schroedinger spectrum - 3 dim';
        lines=1;
        data = inputdlg(entries,title,lines,default,AddOpts);
        
    end    
    
    end
    
        