function [rho, tau, k_range, k_crit]=disordered
% Compute the transmission and reflection amplitudes for a 
% random potential.
%   Lenght=1.e-9 m, 
%   k is the wave number measured in 1e9/m, 
%   mass is in units of electron's mass, 
% the potential is normalized to V_0 times the expression returned by
% the function pot(x), 0<x<1.
% The parameter "multiple barrier" is used to compute the
% amplitudes for a lattice of N equally spaced identical barriers
% at distance L 
%
% Usage:
%   [rho, tau, k_range, k_crit]=multiple;
%   
%  E.Onofri, 2004   

%%% basic units
  
  nano = 1e-9;
  eV   = 1.60217653e-19;		% electron Volt
  hbar = 6.6260693e-34;			% Planck
  m_e  = 9.1093826e-31;			% electron mass
  
  
  %% user interface
  
  selection = {'Resume','New setup'};
  choice    = menu('Start',selection);

  entries = {'Size (L/nm)',...
	     'mass (m/m_e)',...
	     'tol'...
	     'V_0 (eV)',...
	     'random level',...
	     'k (waves/nm)',...
	     'multiple barrier',...
             'interbarrier dist',...
             'save to...'
	   };
  
  switch choice
   case 1				% resume a previous run
    d = dir('*.mat');
    files = {d.name};
    fprompt = {num2cell((1:max(size(files)))'), files'};
    [fprompt{1,:}]
    fn = input('choose number > ');
    eval(['load ' files{fn}]);
    default = setup;
   case 2				% new run
    
    default = {'1.0',... % size
               '1.0',... % mass
               '1.e-12',...
               '1.0',...
	       '0.1',...
               '[0.1:0.1:5.]',...
	       '1',...
               '1.0',...
               'random'
              };
  end
  
  AddOpts.Resize  =  'on';
  AddOpts.WindowStyle  =  'normal';
  AddOpts.Interpreter  =  'tex';
  
  setup = inputdlg(entries, 'parameters', 1, default, AddOpts);
  
  a = nano*str2double(setup{1});		% units nanometers
  mass = str2double(setup{2})*m_e;	% units in electron mass
  tol = str2double(setup{3});
  V0 = eV*str2double(setup{4});	% energy in eV
  nl = nano*str2double(setup{5});
  k_range = str2num(setup{6})/nano;	% units 10^9/meter
  MB = str2double(setup{7});			% how many copies of V(x)?
  L = nano*str2double(setup{8});		% units nanometers
  
  eta = 2*mass*a^2*V0/hbar^2;

  options=odeset('RelTol', tol,...
                 'AbsTol', tol,...
                 'refine',1);
  
  %%% end user interface
  
  
  supp=[0,1];				% support of interaction region
  
  tau=[];
  rho=[];
  h = waitbar(0,'Running ...');
  
  k1=k_range(1);
  k2=k_range(end);
  
  for k=k_range,			% loop on momentum 
    waitbar((k-k1)/(k2-k1));
    ka=k*a;
    uin=[1.0, -i*ka];	% wf and its derivative at x=0
			% propagate from x=0 to x=a using Schrodinger equation
    
    [K,u] = ode113(@schreq, supp, uin, options, ka^2, eta);

    % get end values at x=a and connect to plane waves
    % the phase exp(-ik(L+a)) gives the translation phase shift in
    % case of multiple barriers
    
    uf = u(end,1);
    upf= u(end,2);
    
    sep = 2*nl*(rand(MB,1)-.5);
    
    A = 1/2*(uf - upf/(i*ka))*exp(-i*k*L);
    B = 1/2*(uf + upf/(i*ka))*exp(i*k*L);

    % Now get the reflection and transmision amplitudes  
    % for an array of MB identical barriers 
    
    W1 = [A, conj(B); 
	  B, conj(A)];

    W = W1;
    
    for j=2:MB,
      W = W1 * [exp(-i*k*(sep(j)-sep(j-1))),0;...
		    0,exp(i*k*(sep(j)-sep(j-1)))] * W;
    end    
        
    tau = [tau;1/W(1,1)];
    rho = [rho;W(2,1)/W(1,1)];

  end
  close(h);
  
  % check that unitarity is satisfied 
  err= max(abs((1 - sqrt((abs(tau).^2+abs(rho).^2)))));
  disp(['Error on unitarity  < ', num2str(err)]);

  
  k_crit=sqrt(eta/a^2)*nano;

  subplot(212)
  plot(k_range*nano,abs(tau).^2,'g.-','LineWidth',2);
  hold on  
  plot(k_range*nano,abs(rho).^2,'r.-')
  xlabel('k (waves/nanometer)')
  title('Transmission and reflection coefficients')

  plot(k_crit*[1,1],[0,1],'-.')
  legend('|\tau(k)|^2','|\rho(k)|^2','k_{min}')
  hold off  
  
  subplot(211)
  x=linspace(0,a,128);
  V=V0*pot(x/a);
  k_max=k_range(end);
  E_max=k_max^2/2/mass*hbar^2/eV
  fill([0,x,a]/nano,[0,V,0]/eV,'b')
  axis([0,MB*(a+L)/nano,min(V),E_max])
  ylabel('Pot.Energy/eV')
    title('Potential barrier')
  xlabel('x (nm)')
  hold on
  if MB>1,
    for j=1:MB-1,
      fill(([0,x,a]+j*(a+L)+sep(j))/nano,[0,V,0]/eV,'b')
    end
  end
  hold off
  if(~isempty(setup{9}))
    eval(['save ', setup{9}]);
  end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% Here insert your favourite potential - default = square barrier

function V=pot(x)

%  V = ones(size(x)).*(x<0.1);
 % V = ones(size(x));
%  V = 4.*x.*(1-x);
%  V = 1./cosh(40*(x-1/2)).^2;
  V = 1-2*abs(x-0.5);                   % sawtooth

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% stationary Schrodinger equation 

function  psixx = schreq(x, psi, ka2, eta)

  psixx=[psi(2); (eta*pot(x) - ka2)*psi(1)];
