function [rho, tau, chi, k_range]=multiple
% Compute the transmission and reflection amplitudes for a given
% potential barrier. Units are:
%   Lenght=1.e-9 m, 
%   k is the wave number measured in 1e9/m, 
%   mass is in units of electron's mass, 
% the potential is normalized to V_0 times the expression returned by
% the function pot(x), 0<x<1.
%
% The parameter "multiple barrier" is used to compute the
% amplitudes for a lattice of N equally spaced identical barriers
% at distance L. Where the stability index exceeds one (in absolute
% value) the transmission coefficient goes to zero for large N
% (forbidden band). 
%
% The plot shows: tau and rho (transmission and reflection
% coefficients), chi (the stability index Re(A))
%
% Usage:
%   [rho, tau, chi, k_range]=multiple;
%   
%  E.Onofri, 2004   

%%% basic units
  
  nano = 1e-9;
  eV   = 1.60217653e-19;		% electron Volt
  hbar = 6.6260693e-34;			% Planck
  m_e  = 9.1093826e-31;			% electron mass
  
  
  %% user interface
  
  selection = {'Resume','New setup'};
  choice    = menu('Start',selection);

  entries = {'Size (L/nm)',...
	     'mass (m/m_e)',...
	     'tol'...
	     'V_0 (eV)',...
	     'k (waves/nm)',...
	     'multiple barrier',...
             'interbarrier dist',...
             'save to...'
	   };
  
  switch choice
   case 1				% resume a previous run
    d = dir('*.mat');
    files = {d.name};
    fprompt = {num2cell((1:max(size(files)))'), files'};
    [fprompt{1,:}]
    fn = input('choose number > ');
    eval(['load ' files{fn}]);
    default = setup;
   case 2				% new run
    
    default = {'1.0',... % size
               '1.0',... % mass
               '1.e-12',...
               '1.0',...
               '[0.1:0.1:5.]',...
	       '1',...
               '1.0',...
               'grid'
              };
  end
  
  AddOpts.Resize  =  'on';
  AddOpts.WindowStyle  =  'normal';
  AddOpts.Interpreter  =  'tex';
  
  setup = inputdlg(entries, 'parameters', 1, default, AddOpts);
  
  a = nano*str2num(setup{1});		% units nanometers
  L = nano*str2num(setup{7});		% units nanometers
  mass = str2num(setup{2})*m_e;	% units in electron mass
  tol = str2num(setup{3});
  V0 = eV*str2num(setup{4});	% energy in eV
  k_range = str2num(setup{5})/nano;	% units 10^9/meter
  MB=str2num(setup{6});			% how many copies of V(x)?

  eta = 2*mass*a^2*V0/hbar^2;

  options=odeset('RelTol', tol,...
                 'AbsTol', tol,...
                 'refine',1);
  
  %%% end user interface
  
  
  supp=[0,1];				% support of interaction region
  
  tau=[];
  rho=[];
  chi=[];
  
  h = waitbar(0,'Please wait...');
  
  
  k_min=k_range(1);
  k_max=k_range(end);
  
  
  
  for k=k_range,			% loop on momentum 
    waitbar((k-k_min)/(k_max-k_min));
    ka=k*a;
    uin=[1.0, -i*ka];	% wf and its derivative at x=0
			% propagate from x=0 to x=a using Schrodinger equation
    
    [K,u] = ode113(@schreq, supp, uin, options, ka^2, eta);

    % get end values at x=a and connect to plane waves
    % the phase exp(-ik(L+a)) gives the translation phase shift in
    % case of multiple barriers
    
    uf = u(end,1);
    upf= u(end,2);
  
    A = 1/2*(uf - upf/(i*ka))*exp(-i*k*L);
    B = 1/2*(uf + upf/(i*ka))*exp(i*k*L);

    % Now get the reflection and transmision amplitudes  
    % for an array of MB identical barriers 
    
    W = [A, conj(B); 
	 B, conj(A)];
    
    W = W^MB;				% multiple barriers
    
    W(1,1)=W(1,1)*exp(i*k*(L+a)*MB);
    W(2,1)=W(2,1)*exp(-i*k*(L+a)*MB);
    
    tau = [tau;1/W(1,1)];
    rho = [rho;W(2,1)/W(1,1)];
    chi = [chi; real(A)];
    
  end
  close(h);
  
  % check that unitarity is satisfied 
  err= max(abs((1 - sqrt((abs(tau).^2+abs(rho).^2)))));
  disp(['Error on unitarity  < ', num2str(err)]);

  
  k_crit=sqrt(eta/a^2)*nano;

  plot(k_range*nano,abs(tau).^2,'g.-','LineWidth',2);
  hold on  
  plot(k_range*nano,abs(rho).^2,'r.-')
  plot(k_range*nano,chi.^2,'k')
  xlabel('k (waves/nanometer)')
  title('Transmission and reflection coefficients')
  axis([k_min*nano, k_max*nano, 0, 1.2]);
  plot(k_crit*[1,1],[0,1.2],'-.k')
  
  legend('|\tau(k)|^2','|\rho(k)|^2','\chi^2','k_{tresh}')
  plot([k_min*nano, k_max*nano],[1,1],'--k')
  hold off  

  if(~isempty(setup{8}))
    eval(['save ', setup{8}]);
  end
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% Here insert your favourite potential - default = square barrier

function V=pot(x)

%  V = ones(size(x)).*(x<0.1);
 V = ones(size(x));
%  V = 4.*x.*(1-x);
%  V = 1./cosh(40*(x-1/2)).^2;
%  V = 1-2*abs(x-0.5);                   % sawtooth

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
% stationary Schrodinger equation 

function  psixx = schreq(x, psi, ka2, eta)

  psixx=[psi(2); (eta*pot(x) - ka2)*psi(1)];
