function [set1,set2] = tunneling(set1,set2)
% Wave packets in 1-D under various forms of potential energy
% Export to jpeg for off-line processing 
% A directory ./images contains the images produced during the time evolution
% Usage: 
%   [set1,set2] = wavepkt(set1,set2)
%               if called with no arguments, a default is supplied
% The algorithm is based on Lie-Trotter formula and the fft to go
% to momentum space and back:
% exp(-i t (p^2 + V(q))) ~ IFFT * exp(-i t p^2) * FFT * exp(-it V(q))
% set checkT='y' to see the time inversion
% set save image to 'y' to record the snapshots
% © E.Onofri 2013
   
%--------------    Begin user interface --------------
   entr1 = {'Size',...
            'Grid',...
            'mass',...
            'time',...
            'Snapshots',...
            'EraseMode',...
            'mom_{Init}',...
            'x_0',...
            '\sigma'
            };
   entr2 = {'potential strength',...
            'movie prefix',...
            'refine',...
            'pot_{param}',...
            'save image',...
            'check T',...
            'n-peaks',...
            'tilt'
           };
   
   if(nargin<2)
      def2 = {'4.0',...
              'tunnel_',...
              '16',...
              '4.0',...
              'n',...
              'n',...
              '1',...
              '0.00'
             };
   else
      def2 = set2;
   end
   
   if(nargin<1)
            def1 = {'200.0',... % Size
              '2048',...  % NGrid
              '1.0',...  % mass
              '120.0',...   % time
              '512',...   % snapshots
              'normal',...   % EraseMode 
              '2.0',...
              '-100.0',...
              '256.0',...
             };
   else
      def1 = set1;
   end
   
   heading = 'Initial data';
   lines = 1;
   AddOpts.Resize  =  'on';
   AddOpts.WindowStyle = 'normal';
   AddOpts.Interpreter = 'tex';
   
   set1 = inputdlg(entr1,heading,lines,def1,AddOpts);
   set2 = inputdlg(entr2,heading,lines,def2,AddOpts);
   
   L          = str2double(set1{1});
   N          = str2num(set1{2});
   mass       = str2double(set1{3});
   Tmax       = str2double(set1{4});
   snapshots  = str2num(set1{5});
   plotmode   = set1{6};
   mom        = str2double(set1{7});
   x0         = str2double(set1{8});
   sigma      = str2double(set1{9});
   V0         = str2double(set2{1})
   moviename  = set2{2};
   refine     = str2double(set2{3})
   wdth       = str2double(set2{4});
   modes      = [{'normal'},{'xor'},{'none'}];
   save_img   = set2{5};
   checkT     = set2{6};
   n_peaks    = str2num(set2{7});
   tilt       = str2num(set2{8});
   
   if(nnz(strcmp(modes,plotmode))==0),
        disp(['Unknown mode: ' plotmode]);
        disp('Legal modes: ')
        disp(modes);
        plotmode = 'normal';
    end
    
%--- end user interface ----------
    
%--- organize the plot -----------

    scrsz = get(0,'ScreenSize');
    fig = figure('Position',[1, 1, scrsz(3)/2, scrsz(4)/2]);
    
    set(fig,'DoubleBuffer','on','Windowstyle','docked');
    set(gca,'NextPlot','replace','Visible','off')

    %%-----------------  coordinate grid
  
    dx  = 2*L/N
    x   = -L*(1-1/N) : dx : L*(1-1/N); 
    x   =  x';

    %%------------------  momentum space grid
    
    k = fftshift(pi*(-N/2:(N/2-1))/L)';
    Ken = k.^2/(2*mass);
    tau = Tmax/snapshots;
    t = tau;
    Ken = min(Ken, pi *refine/tau);  % anti-alias cutoff
    
    %
    %%------------------ Insert potential here 
    %
    % V = tanh((x+wdth)/4)-tanh((x-wdth)/4);
    % for n=2:n_peaks,
    %    V = V + tanh((x-(4*n-5)*wdth)/4) - tanh((x-(4*n-3)*wdth)/4);
    % end
    % V = V*V0/2;
    
    %    V = V0 * (abs(x)<wdth);      %  square barrier
    % V = 0.5*V0*mass*wdth^2*x.^2; %  harmonic oscillator
    % V = V0*mass*wdth*abs(x);     %  constant force 
    % V = V0 * (x.^2 - wdth^2).^2/wdth^4; 
    V = V0 ./ cosh((x-L/4)./wdth).^2; % 1/cosh - "reflectionless potential"
    V = V + max(x,0)*tilt;
    
    V = min(V, pi*refine/tau);          % "anti-aliasing"
    U = exp(-i*tau*V/refine);    
    U2 = exp(-0.5i*tau*V/refine); % half step 
    W = exp(-i*Ken*tau/refine);
    
    %prepare a normalized initial wfcn
    psi = exp(-(x-x0).^2/4/sigma+i*mom*x)/(2*pi*sigma).^(1/4);
    rho = abs(psi).^2;

    area(x,V/abs(V0)*max(rho)/2,'Facecolor','y',...
         'EraseMode',plotmode,...
         'LineStyle','none');

    hold on

    plot(x, rho, 'w:');
    axis tight
    
    xlabel('x'); 
    ylabel('|\Psi(x,t)|^2');
    title(['Tunneling']);

    set(gca,'Color',    [.64,.65,.99])
    set(gcf,'DoubleBuffer','on');

    h = area(x, rho,'Facecolor', 'b',...
           'EraseMode',plotmode,...
           'LineStyle','none');
    
    h1 = findobj(gca,'Type','patch');
    set(h1,'FaceAlpha',0.5);

    % compute kinetic energy and its standard deviation
    
    phi = fft(psi);
    E = sum(Ken.*abs(phi).^2)/sum(abs(phi).^2);
    deltaE = sum(Ken.^2.*abs(phi).^2)/sum(abs(phi).^2)- ...
              E^2;
    deltaE = sqrt(deltaE);

    Emax = (E+deltaE)/abs(V0)*max(rho)/2; % scale is the same of V(x)
    Emin = (E-deltaE)/abs(V0)*max(rho)/2;
    E    = E/abs(V0)*max(rho)/2;
              
    plot([x(1),x(end)],Emax*[1,1],'-.c',...
         [x(1),x(end)],Emin*[1,1],'-.c')
    plot([x(1),x(end)],E*[1,1],'c','LineWidth',2);

    whitebg([.6,.6,.6]);
    
    % main loop
    n = 0;
    
    while(t<Tmax),
        set(h,'Ydata',abs(psi).^2);
        drawnow
        psi=evolve(psi);
        home;
        disp(['time:', num2str(t)]);
        t = t+tau ;
        if lower(save_img)=='y',        % capture snapshots
           n = n+1;
           ver = sprintf('%d',1000000+n);
           print('-djpeg75',['images/',moviename,ver]);
        end
    end
    
    if(lower(checkT)=='y'),             % time reversal
       t = tau;
       psi = conj(psi);
       while(t<Tmax),
          psi=evolve(psi);
          home;
          set(h,'Ydata',abs(psi).^2);
          drawnow
          disp(['time:', num2str(t)]);
          t = t+tau ;
       end
    end
    hold off;

function Psi=evolve(psi)
   psi = psi.*U2;
   for j = 1:refine                  % U2*W*U*W*U*....*W*U/U2 
      phi = fft(psi);
      psi = U.*ifft(phi.*W);
   end
   Psi = psi.*conj(U2);
end
end

    